'''
esy.osmfilter
===========

Filter for  OpenStreetMap Protobuf data (aka `.pbf` files).

For convenience, the toplevel module :mod:`esy.osmfilter` links to the most
relevant classes and functions of this library:

.. autosummary::
    :nosignatures:

    ~esy.osmfilter.osm_filter.run_filter
'''

from . import  osm_colors          as CC
from . osm_filter import run_filter as run_filter
from esy.osm.pbf import Node, Way,Relation


